/***************************************************************************************************

Simplest possible SEPARATE help system (that is reasonable). Must have: 
	* table to hold answers
	* sp to insert answers to table 
	* sp to search those answers

Use: Great for administrative systems, in-house applications, pulished code.
	 Think: Centralized help library.

***************************************************************************************************/
USE master;
GO

CREATE SCHEMA HELP;
GO

CREATE TABLE HELP.Answer
       (
         ID INT IDENTITY(1, 1)
       , Title VARCHAR(200) NOT NULL	-- Identifying title
       , Article VARCHAR(MAX) NOT NULL	-- The body of the answer
       );

GO

CREATE PROCEDURE HELP.AnswerInsert
       @Title VARCHAR(200)
     , @Article VARCHAR(MAX)
AS
/******************************************************************************************
	Purpose: Used manually to insert new "Help" topics to the HELP.Answer table. 
		Part of a home grown help system used by CompanyX database administrators.

	Author: Jen McCown, Jen@MidnightDBA.com

	Parameters:
		@Title - Identifying title, like "How to insert a row to HELP.Answer".
		@Article - The body of the HELP answer.

	Example execution: 
		EXEC HELP.AnswerInsert 'How to do things', 'Try really hard.';
	
		EXEC HELP.AnswerInsert 'HELP.AnswerInsert', 'Procedure - for the DBA inline help system.' 
	
		EXEC HELP.AnswerInsert 'dbo.DoExampleThings', 'Procedure to exemplify the @help param.
	Be sure to look at the SP code itself for full documentation.';

	Created: 8/13/2016
******************************************************************************************/
       BEGIN
             INSERT INTO HELP.Answer
                    ( Title, Article )
             VALUES ( @Title, @Article );
       END;
GO



---- Insert rows: 
		EXEC HELP.AnswerInsert 'How to do things', 'Try really hard.';
	
		EXEC HELP.AnswerInsert 'HELP.AnswerInsert', 'Procedure - for the DBA inline help system.' 
	
		EXEC HELP.AnswerInsert 'dbo.DoExampleThings', 'Procedure to exemplify the @help param.
	Be sure to look at the SP code itself for full documentation.';
GO
---- Create procedure HELP.Me: 

CREATE PROCEDURE HELP.Me
       @Title VARCHAR(200) = '%'
AS
/******************************************************************************************
Purpose: Used to return help topics from the HELP.Answer table. 

Author: Jen McCown, Jen@MidnightDBA.com

Parameters:
	@Title - Title to search for, like "How to insert a row to HELP.Answer". You can use 
		LIKE wildcards (such as % and _) to search for titles.
	
Discussion: 
	I'd like to make this better; the LIKE wildcards are fine, but we can do more.
	Like, giving the option to search the answer bodies; and/or, returning a list of
	available topics.

Example execution: 
	EXEC HELP.Me 'How to do things';

	EXEC HELP.Me 'HELP';
	
	EXEC HELP.Me;

Created: 8/13/2016

******************************************************************************************/
       BEGIN

             IF @Title IS NULL
                SET @Title = '%';


             SELECT Title
                  , Article
             FROM   HELP.Answer
             WHERE  Title LIKE '%' + @Title + '%';
       END;
GO




/**************************************************************************************************
***************************************************************************************************

How could we make this better? Provide the ability to: 
	* divide answers into major sections
	* mark answer types: object, how to, procedure, etc.
	* make the TEXT of the answers searchable, too
	* record the answer's author
	* record the date the answer was last inserted, updated
	* mark answers as relevant to other answers
	* group and order answers, to make a sort of document
	* access help FROM WITHIN SPs, via a @help=1 parameter

Good ideas:
	Look at existing help systems and how they make sense. Powershell's get-help is wonderful; 
	articles are divided up into sections, and offer a topic name, short desc, long desc, links, 
	examples, and a SEE ALSO section.

***************************************************************************************************
***************************************************************************************************/


---------------------------------------------------------------------------
-- CHALLENGE: Provide the ability to access help FROM WITHIN SPs, via a @help=1 parameter. Easy.

CREATE PROCEDURE dbo.DoExampleThings
      @param INT = 1
    , @help BIT = 0

/* 
 Purpose: This is a fake SP meant only to demo things in SQL Saturday sessions.

 Example execution: 
	EXEC dbo.DoExampleThings 1, 0;
	EXEC dbo.DoExampleThings @help=1;
*/
AS
BEGIN
    IF @help = 1
        BEGIN	-- IF @help = 1

            ---- I would turn this whole thing into a function, I think: 
			DECLARE @procname NVARCHAR(256);
            SELECT @procname = name
            FROM   sys.procedures
            WHERE  object_id = @@PROCID;
		
            EXEC HELP.Me @Title = @procname;
		
            RETURN;
        END;	-- IF @help = 1
    
	------------------------------- BEGIN Do Something ------------------------------- 
    IF @param IS NOT NULL
    SELECT  @@PROCID AS DidSomething;
    ELSE
    SELECT  0 AS DidSomething;

	------------------------------- END Do Something ------------------------------- 
        
END;
GO




/*
-- Demo cleanup: 

USE master;
GO
DROP PROCEDURE dbo.DoExampleThings;
GO
DROP PROCEDURE HELP.Me;
GO
DROP PROCEDURE HELP.AnswerInsert;
GO
DROP TABLE HELP.Answer;
GO
DROP SCHEMA HELP;

-- */